/*
 * Copyright (c) 2005-2008 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package fr.obeo.acceleo.gen.profiler.presentation;

import fr.obeo.acceleo.gen.profiler.LoopProfileEntry;
import fr.obeo.acceleo.gen.profiler.ProfileEntry;

import java.util.Comparator;
import java.util.Set;
import java.util.TreeSet;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;

/**
 * Content provider for the profiler outline.
 * 
 * @author Yvan LUSSAUD <a
 *         href="mailto:yvan.lussaud@obeo.fr">yvan.lussaud@obeo.fr</a>
 * 
 */
public class ProfilerOutlineContentProvider extends AdapterFactoryContentProvider {

	/**
	 * Constructor.
	 * 
	 * @param adapterFactory
	 *            the adapter factory
	 */
	public ProfilerOutlineContentProvider(AdapterFactory adapterFactory) {
		super(adapterFactory);
	}

	/**
	 * {@inheritDoc}
	 */
	public Object[] getChildren(Object object) {
		if (object instanceof LoopProfileEntry) {
			final Set sorted = new TreeSet(new Comparator() {
				public int compare(Object o1, Object o2) {
					long diff = ((ProfileEntry) o2).getDuration() - ((ProfileEntry) o1).getDuration();
					if (diff != 0) {
						return (int) diff;
					} else {
						return ((ProfileEntry) o1).getMonitored().compareTo(((ProfileEntry) o2).getMonitored());
					}
				}
			});

			sorted.addAll(((LoopProfileEntry) object).getLoopElements());
			return sorted.toArray();
		} else {
			return new Object[] {};
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean hasChildren(Object object) {
		return object instanceof LoopProfileEntry && ((LoopProfileEntry) object).getLoopElements().size() > 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public Object[] getElements(Object object) {
		return getChildren(object);
	}
}
